/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonIcon.h"

extern UINT (WINAPI *pPrivateExtractIcons)(LPCTSTR,INT,INT,INT,HICON*,UINT*,UINT,UINT);

ButtonIcon::ButtonIcon()
{
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::subconfigure(const string &uniquetype, const string &grouptype, const string &prefixkey, ICONSETTINGS &workSettings, ICONSETTINGS &defaultSettings)
{
	workSettings.showIcon = Getx2RCBool(uniquetype, grouptype, (prefixkey + "ShowIcon"), defaultSettings.showIcon);
	workSettings.bigIcon = Getx2RCBool(uniquetype, grouptype, (prefixkey + "UseBigIcon"), defaultSettings.bigIcon);
	workSettings.iconSize = Getx2RCInt(uniquetype, grouptype, (prefixkey + "IconSize"), defaultSettings.iconSize, 8, 64);
	workSettings.iconX = Getx2RCString(uniquetype, grouptype, (prefixkey + "IconX"), defaultSettings.iconX);
	if ( defaulticon.iconY.empty() )
		workSettings.iconY = Getx2RCString(uniquetype, grouptype, (prefixkey + "IconY"), ("-" + IntToString(workSettings.iconSize/2) + "c"));
	else
		workSettings.iconY = Getx2RCString(uniquetype, grouptype, (prefixkey + "IconY"), defaulticon.iconY);
	workSettings.icon_hueColor = Getx2RCColor(uniquetype, grouptype, (prefixkey + "IconHueColor"), defaultSettings.icon_hueColor);
	workSettings.icon_hueIntensity = Getx2RCInt(uniquetype, grouptype, (prefixkey + "IconHueIntensity"), defaultSettings.icon_hueIntensity, 0, 100);
	workSettings.icon_luminanceIntensity = Getx2RCInt(uniquetype, grouptype, (prefixkey + "IconLuminanceIntensity"), defaultSettings.icon_luminanceIntensity, -100, 100);
	workSettings.icon_saturationIntensity = Getx2RCInt(uniquetype, grouptype, (prefixkey + "IconSaturationIntensity"), defaultSettings.icon_saturationIntensity, 0, 200);

	workSettings.icon_mixColor = Getx2RCColor(uniquetype, grouptype, (prefixkey + "IconMixColor"), defaultSettings.icon_mixColor);
	workSettings.icon_mixIntensity = Getx2RCInt(uniquetype, grouptype, (prefixkey + "IconMixIntensity"), defaultSettings.icon_mixIntensity, 0, 100);
}

void ButtonIcon::configure(const string &uniquekey, const string &groupkey)
{
	defaulticon.showIcon = true;
	defaulticon.bigIcon = false;
	defaulticon.iconSize = 16;
	defaulticon.iconX = "0";
	defaulticon.iconY = "";

	defaulticon.icon_hueColor = RGB(255, 255, 255);
	defaulticon.icon_hueIntensity = 0;
	defaulticon.icon_luminanceIntensity = 0;
	defaulticon.icon_saturationIntensity = 100;

	defaulticon.icon_mixColor = RGB(255, 255, 255);
	defaulticon.icon_mixIntensity = 0;

	subconfigure(uniquekey, groupkey, "Title", title, defaulticon);
	title.showIcon = Getx2RCBool(uniquekey, groupkey, "TitleShowIcon", false);

	subconfigure(uniquekey, groupkey, "Entry", entry, defaulticon);

	subconfigure(uniquekey, groupkey, "ActiveEntry", selentry, entry);

	subconfigure(uniquekey, groupkey, "Folder", folder, entry);

	subconfigure(uniquekey, groupkey, "ActiveFolder", selfolder, (Getx2RCBool(uniquekey, groupkey, "ActiveFolderUseFolderDefault") ? folder : selentry));

	subconfigure(uniquekey, groupkey, "Pinned", pinned, defaulticon);
	pinned.showIcon = Getx2RCBool(uniquekey, groupkey, "PinnedShowIcon", false);
	pinnedIconPath = Getx2RCString(uniquekey, groupkey, "PinnedIconPath", "");
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, const string &iconPath, const HWND iconhWnd, const LPITEMIDLIST iconpidl, const string &DefaultIcon, bool useAlphaMap)
{
	if ( !IsOS( OS_2KXP ) )
		useAlphaMap = false;

	int iconX = 0;
	int iconY = 0;
	int iconSize = 16;
	bool bigIcon = false;

	COLORREF _hueColor = RGB(255, 255, 255);
	int _hueIntensity = 0;
	int _luminanceIntensity = 0;
	int _saturationIntensity = 100;

	COLORREF _mixColor = RGB(255, 255, 255);
	int _mixIntensity = 0;

	if (buttonType == -1)
	{
		if (!pinned.showIcon)
			return;
		iconX = xParseCoord(pinned.iconX, 0, width, 0);
		iconY = xParseCoord(pinned.iconY, 0, height, 0);

		iconSize = pinned.iconSize;
		bigIcon = pinned.bigIcon;

		_hueColor = pinned.icon_hueColor;
		_hueIntensity = pinned.icon_hueIntensity;
		_luminanceIntensity = pinned.icon_luminanceIntensity;
		_saturationIntensity = pinned.icon_saturationIntensity;

		_mixColor = pinned.icon_mixColor;
		_mixIntensity = pinned.icon_mixIntensity;
	}
	else if (buttonType == 0)
	{
		if (!title.showIcon)
			return;
		iconX = xParseCoord(title.iconX, 0, width, 0);
		iconY = xParseCoord(title.iconY, 0, height, 0);

		iconSize = title.iconSize;
		bigIcon = title.bigIcon;
	
		_hueColor = title.icon_hueColor;
		_hueIntensity = title.icon_hueIntensity;
		_luminanceIntensity = title.icon_luminanceIntensity;
		_saturationIntensity = title.icon_saturationIntensity;
		
		_mixColor = title.icon_mixColor;
		_mixIntensity = title.icon_mixIntensity;
	}
	else if (buttonType == 1)
	{
		if (!entry.showIcon)
			return;
		iconX = xParseCoord(entry.iconX, 0, width, 0);
		iconY = xParseCoord(entry.iconY, 0, height, 0);

		iconSize = entry.iconSize;
		bigIcon = entry.bigIcon;
	
		_hueColor = entry.icon_hueColor;
		_hueIntensity = entry.icon_hueIntensity;
		_luminanceIntensity = entry.icon_luminanceIntensity;
		_saturationIntensity = entry.icon_saturationIntensity;

		_mixColor = entry.icon_mixColor;
		_mixIntensity = entry.icon_mixIntensity;
	}
	else if (buttonType == 2)
	{
		if (!selentry.showIcon)
			return;
		iconX = xParseCoord(selentry.iconX, 0, width, 0);
		iconY = xParseCoord(selentry.iconY, 0, height, 0);

		iconSize = selentry.iconSize;
		bigIcon = selentry.bigIcon;
	
		_hueColor = selentry.icon_hueColor;
		_hueIntensity = selentry.icon_hueIntensity;
		_luminanceIntensity = selentry.icon_luminanceIntensity;
		_saturationIntensity = selentry.icon_saturationIntensity;

		_mixColor = selentry.icon_mixColor;
		_mixIntensity = selentry.icon_mixIntensity;
	}
	else if (buttonType == 3)
	{
		if (!folder.showIcon)
			return;
		iconX = xParseCoord(folder.iconX, 0, width, 0);
		iconY = xParseCoord(folder.iconY, 0, height, 0);

		iconSize = folder.iconSize;
		bigIcon = folder.bigIcon;
	
		_hueColor = folder.icon_hueColor;
		_hueIntensity = folder.icon_hueIntensity;
		_luminanceIntensity = folder.icon_luminanceIntensity;
		_saturationIntensity = folder.icon_saturationIntensity;

		_mixColor = folder.icon_mixColor;
		_mixIntensity = folder.icon_mixIntensity;
	}
	else if (buttonType == 4)
	{
		if (!selfolder.showIcon)
			return;
		iconX = xParseCoord(selfolder.iconX, 0, width, 0);
		iconY = xParseCoord(selfolder.iconY, 0, height, 0);

		iconSize = selfolder.iconSize;
		bigIcon = selfolder.bigIcon;
	
		_hueColor = selfolder.icon_hueColor;
		_hueIntensity = selfolder.icon_hueIntensity;
		_luminanceIntensity = selfolder.icon_luminanceIntensity;
		_saturationIntensity = selfolder.icon_saturationIntensity;

		_mixColor = selfolder.icon_mixColor;
		_mixIntensity = selfolder.icon_mixIntensity;
	}

	HICON hIcon = NULL;
	if ( buttonType == -1 && !pinnedIconPath.empty() )
	{
		hIcon = GetIconFromPathEx(pinnedIconPath.c_str(), bigIcon);
	}
	else if ( !iconPath.empty() )
	{
		hIcon = GetIconFromPathEx(iconPath.c_str(), bigIcon);
	}
	else if ( iconhWnd != NULL )
	{
		hIcon = GetTaskIcon(iconhWnd, bigIcon);
	}
	else if ( iconpidl != NULL )
	{
		SHFILEINFO shfi;
		SHGetFileInfo((LPCSTR) iconpidl, 0, &shfi, sizeof(SHFILEINFO), SHGFI_PIDL | SHGFI_DISPLAYNAME | SHGFI_ICONLOCATION | (bigIcon ? SHGFI_LARGEICON : SHGFI_SMALLICON) | SHGFI_ICON);
		// Extract the icon
		if ( pPrivateExtractIcons != NULL )
			pPrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, (bigIcon ? 32 : 16), (bigIcon ? 32 : 16), &hIcon, NULL, 1, NULL);

		if (hIcon == NULL)
			hIcon = shfi.hIcon;
		else
			DestroyIcon(shfi.hIcon);
	}
	else 
	{
		return;
	}

	if ( hIcon == NULL && buttonType == -1 )
		return;

	if ( hIcon == NULL && _stricmp(iconPath.c_str(), ".none") != NULL )
		hIcon = GetIconFromPathEx(DefaultIcon.c_str(), bigIcon);
	
	if ( hIcon == NULL )
		return;

	if ( useAlphaMap || _hueIntensity != 0 || _luminanceIntensity != 0 || _saturationIntensity != 100 || _mixIntensity != 0 )
	{
		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=iconSize;      // Set size you need
		alpha.bmiHeader.biHeight=iconSize;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;

		HDC hdcBGCOPY = CreateCompatibleDC(NULL);
		HBITMAP	hbmBGCOPY = CreateDIBSection( hdcBGCOPY,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );
		HBITMAP oldBGCOPYBMP = (HBITMAP) SelectObject(hdcBGCOPY, hbmBGCOPY);

		//Copy RealBG in Result Background
		BitBlt( hdcBGCOPY, 0, 0, iconSize, iconSize, hDC, x+iconX, y+iconY, SRCCOPY );

		HDC hdcBuffer = CreateCompatibleDC(NULL);
		HBITMAP	hbmBuffer = CreateDIBSection( hdcBuffer,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );
		HBITMAP oldbufferBMP = (HBITMAP) SelectObject(hdcBuffer, hbmBuffer);

		//Copy RealBG in Result Background
		if ( IsOS(OS_2KXP) )
			BitBlt(hdcBuffer, 0, 0, iconSize, iconSize, hDC, x+iconX, y+iconY, SRCCOPY );

		//Draw Icon
		DrawIconEx(hdcBuffer, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL );

		HDC hdcMask = CreateCompatibleDC(NULL);
		HBITMAP	hbmMask = CreateDIBSection( hdcMask,
											&alpha,
											DIB_RGB_COLORS,
											&pvBits,
											NULL,
											0 );
		HBITMAP oldmaskBMP = (HBITMAP) SelectObject(hdcMask, hbmMask);

		//Draw Mask of the Icon
		DrawIconEx(hdcMask, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

		if ( IsOS(OS_XP) )
		{
			BITMAP bm;
			GetObject(hbmBuffer, sizeof(BITMAP), &bm);

			BITMAP bmmask;
			GetObject(hbmMask, sizeof(BITMAP), &bmmask);

			BITMAP bmorg;
			GetObject(hbmBGCOPY, sizeof(BITMAP), &bmorg);

			BYTE *pPixel = (LPBYTE)bm.bmBits;
			BYTE *pPixelMask = (LPBYTE)bmmask.bmBits;
			BYTE *pPixelOrg = (LPBYTE)bmorg.bmBits;

			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;

			BYTE h = 0;
			BYTE s = 0;
			BYTE l = 0;
			
			BYTE hueh = 0;
			BYTE hues = 0;
			BYTE huel = 0;

			bool needHSLConversion = false;
			bool needMixColor = false;

			if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
				needHSLConversion = true;

			if ( _hueIntensity != 0 )
			{
				r = GetRValue(_hueColor);
				g = GetGValue(_hueColor);
				b = GetBValue(_hueColor);

				if ( r == b && b == g )
				{
					_hueIntensity = 0;
				}
				else
				{
					RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
					needHSLConversion = true;
				}
			}

			if ( _mixIntensity != 0 )
			{
				_mixIntensity = (int) min(255, _mixIntensity*2.55);
				needMixColor = true;
			}

			bool skip = false;

			int iSize = bm.bmHeight * bm.bmWidth;

			for (int i = 0; i < iSize; ++i)
			{
				//Filter Mask and insert OriginalBG to preserve AlphaChannel
				if ( pPixelMask[0] == 255 && pPixelMask[1] == 255 && pPixelMask[2] == 255 )
				{
					pPixel[0] = pPixelOrg[0];	
					pPixel[1] = pPixelOrg[1];	
					pPixel[2] = pPixelOrg[2];	
					pPixel[3] = pPixelOrg[3];	

					skip = true;
				}
				//For colorizing without AlphaMap
				if ( !skip && pPixelOrg[0] == 255 && pPixelOrg[1] == 0 && pPixelOrg[2] == 255 && pPixel[3] != 255 && pPixel[3] != 0 )
				{
					pPixel[0] = pPixelOrg[0];	
					pPixel[1] = pPixelOrg[1];	
					pPixel[2] = pPixelOrg[2];	
					pPixel[3] = pPixelOrg[3];	

					skip = true;
				}

				if ( skip )
				{
					pPixel+= 4;
					pPixelMask+= 4;
					pPixelOrg+= 4;

					skip = false;
					continue;
				}

				r = pPixel[0];
				g = pPixel[1];
				b = pPixel[2];

				if ( needHSLConversion )
				{
					//AlphaMap Colorizing
					RGB2HSL(r, g, b, h, s, l);

					if(_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
				
					if(_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

					if(_saturationIntensity != 100)
					{
						if ( _saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
					}

					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix Effect
					if ( pPixel[3] != 0 )
					{
						r = (BYTE)((r * (255 - _mixIntensity) + (pPixel[3]*GetBValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _mixIntensity) + (pPixel[3]*GetGValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _mixIntensity) + (pPixel[3]*GetRValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
					}
					else
					{
						r = (BYTE)((r * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
					}
				}

				pPixel[0] = r;
				pPixel[1] = g;
				pPixel[2] = b;
				//Fix for Icons without AlphaChannel (all is 0!), with AlphaChannel 0 is Mask
				if ( pPixel[3] == 0 )
					pPixel[3] = 255;

				pPixel+= 4;
				pPixelMask+= 4;
				pPixelOrg+= 4;
			}

			BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBuffer, 0, 0, SRCCOPY);
		}
		else
		{
			int iy = 0;
			int ix = 0;
			COLORREF cl = 0;
			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;

			BYTE h = 0;
			BYTE s = 0;
			BYTE l = 0;
			
			BYTE hueh = 0;
			BYTE hues = 0;
			BYTE huel = 0;

			bool needHSLConversion = false;
			bool needMixColor = false;

			if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
				needHSLConversion = true;

			if ( _hueIntensity != 0 )
			{
				r = GetRValue(_hueColor);
				g = GetGValue(_hueColor);
				b = GetBValue(_hueColor);

				if ( r == b && b == g )
				{
					_hueIntensity = 0;
				}
				else
				{
					RGB2HSL(r, g, b, hueh, hues, huel);
					needHSLConversion = true;
				}
			}

			if ( _mixIntensity != 0 )
			{
				_mixIntensity = (int) min(255, _mixIntensity*2.55);
				needMixColor = true;
			}

			for(iy = 0; iy < iconSize; iy++)
			{
				for(ix = 0; ix < iconSize; ix++)
				{
					// Loop through all transparent pixels...
					// If the mask-pixel is white or grey(if that is possible?), then break
					if( !GetPixel(hdcMask, ix, iy) )
					{
						cl = GetPixel(hdcBuffer, ix, iy);
						r = GetRValue(cl);
						g = GetGValue(cl);
						b = GetBValue(cl);

						if ( needHSLConversion )
						{
							RGB2HSL(r, g, b, h, s, l);

							if(_hueIntensity != 0)
								h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
				
							if(_luminanceIntensity != 0)
								l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

							if(_saturationIntensity != 100)
							{
								if ( _saturationIntensity > 100 )
									s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
								else
									s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
							}
						
							HSL2RGB(r, g, b, h, s, l);
						}

						if ( needMixColor )
						{
							// HueMix Effect
							r = (BYTE)((r * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
							g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
							b = (BYTE)((b * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
						}

						SetPixelV(hdcBGCOPY, ix, iy, RGB(r,g,b));
					}
				}
			}

			BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBGCOPY, 0, 0, SRCCOPY);
		}
				
		hbmMask = (HBITMAP) SelectObject(hdcMask, oldmaskBMP);
		DeleteObject(hbmMask);
		DeleteDC(hdcMask);
		DeleteObject(oldmaskBMP);

		hbmBuffer = (HBITMAP) SelectObject(hdcBuffer, oldbufferBMP);
		DeleteObject(hbmBuffer);
		DeleteDC(hdcBuffer);
		DeleteObject(oldbufferBMP);

		hbmBGCOPY = (HBITMAP) SelectObject(hdcBGCOPY, oldBGCOPYBMP);
		DeleteObject(hbmBGCOPY);
		DeleteDC(hdcBGCOPY);
		DeleteObject(oldBGCOPYBMP);
	}
	else
	{
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);
	}

	DestroyIcon(hIcon);
}
