/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_POPUPTEXTURE_H__B67B692C_6626_4381_A330_138EF9041D56__INCLUDED_)
#define AFX_POPUPTEXTURE_H__B67B692C_6626_4381_A330_138EF9041D56__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef vector<COLORREF> ColorRefList;
typedef vector<COLORREF>::iterator ColorRefListIterator;

#define	GRADIENT_NONE			-1
#define	GRADIENT_HORIZONTAL		0
#define	GRADIENT_VERTICAL		1
#define	GRADIENT_RADIAL			2
#define	GRADIENT_DIAGONAL		3
#define	GRADIENT_FDIAGONAL		4
#define	GRADIENT_BDIAGONAL		5

#define	TRANSFORMATION_NONE			-1
#define	TRANSFORMATION_CHARICATURE  0
#define	TRANSFORMATION_FISHEYE		1
#define	TRANSFORMATION_SWIRLED		2
#define	TRANSFORMATION_CYLINDER		3
#define	TRANSFORMATION_SHIFT		4

class PopupTexture  
{
public:
	PopupTexture();
	virtual ~PopupTexture();

	void configure(const string &uniquekey, const string &groupkey, bool alphaMap);
	void apply(HDC hDC, int x, int y, int width, int height);
	void applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaTransparency);

	int getMinNeededHeight() { return (topEdge+bottomEdge); };

private:

	bool useAlphaMap;

	bool useSolidColors;

	void parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor);
	void parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel );
	void solidGradientPaint(HDC destHDC, RECT r);
	void paintEffects(HDC destHDC, int width, int height);

	unsigned char* m_Data;
	void DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType);
	void ApplyTransformation(int width, int height, int transformationType);

	void PrepareAlphaChannel(HBITMAP &bmp);

	HDC memoryDC;
	HBITMAP memoryBitmap;

	string solidColors;

	ColorRefList Gradient;

	int gradientType;
	int transformationType;

	int imageHeight;
	int imageWidth;

	int borderMethod;

	int leftEdge;
	int topEdge;
	int rightEdge;
	int bottomEdge;

	int mode;

	COLORREF popup_hueColor;
	int popup_hueIntensity;
	int popup_luminanceIntensity;
	int popup_saturationIntensity;

	COLORREF popup_mixColor;
	int popup_mixIntensity;

	int bevelSizeLeft, bevelSizeRight, bevelSizeTop, bevelSizeBottom;

	bool verticalGradient;
	int  repeatedGradient;

	COLORREF backgroundColor;
	COLORREF darkBevelColor;
	COLORREF lightBevelColor;
};

#endif // !defined(AFX_POPUPTEXTURE_H__B67B692C_6626_4381_A330_138EF9041D56__INCLUDED_)
